/*
 * Copyright 2007 by Kappich Systemberatung Aachen 
 * 
 * This file is part of de.bsvrz.dav.daf.
 * 
 * de.bsvrz.dav.daf is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.dav.daf is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with de.bsvrz.dav.daf; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package de.bsvrz.dav.daf.main.config.management;

import de.bsvrz.dav.daf.main.DataAndATGUsageInformation;
import de.bsvrz.dav.daf.main.config.ConfigurationTaskException;
import de.bsvrz.dav.daf.main.config.MutableCollectionChangeListener;
import de.bsvrz.dav.daf.main.config.SystemObject;

import java.util.Collection;
import java.util.List;

/**
 * Die Implementation dieses Interfaces erlaubt es, die Benutzer der Kernsoftware zu verwalten. Dies beinhaltet Aktionen wie:<br> <ul> <li>Anlegen neuer
 * Benutzer</li> <li>ndern von Passwrtern</li> <li>ndern der Rechte, die ein Benutzer besitzt</li> <li>Erstellung von Einmal-Passwrtern</li> </ul>
 * <p/>
 * Alle beschriebenen Aktionen setzen dabei die ntigen Rechte des Benutzers voraus, der die Aktion auslst. Sind die ntigen Rechte nicht vorhanden, so wird
 * die Aktion nicht durchgefhrt.
 *
 * @author Kappich Systemberatung
 * @version $Revision: 8238 $
 */
public interface UserAdministration {

	/**
	 * Beauftragt die Konfiguration ein neues Benutzerkonto anzulegen. Zu einem Benutzerkonto gehren ein Benutzername, ein Passwort und die Rechte des Benutzers.
	 * <p/>
	 * Ein neuer Benutzer kann nur durch einen Benutzer angelegt werden, der die Rechte eines Administrators besitzt. Besitzt der Benutzer diese Rechte nicht, wird
	 * der Auftrag zur Konfiguration bertragen und dort abgelehnt.
	 *
	 * @param orderer              Benutzername des Auftraggebers.
	 * @param ordererPassword      Passwort des Auftraggebers.
	 * @param newUsername          Benutzername des neuen Benutzers.
	 * @param newUserPid           Pid des neuen Benutzers. Wird der Leerstring ("") bergeben, wird dem Benutzer keine explizite Pid zugewiesen.
	 * @param newPassword          Passwort des neuen Benutzers.
	 * @param adminRights          <code>true</code>, wenn der neue Benutzer die Rechte eines Administrators besitzen soll; <code>false</code>, wenn der Benutzer
	 *                             keine speziellen Rechte besitzen soll.
	 * @param pidConfigurationArea Pid des Konfigurationsbereichs, in dem der neue Benutzer angelegt werden soll.
	 *
	 * @throws de.bsvrz.dav.daf.main.config.ConfigurationTaskException
	 *          Die Konfiguration kann den Auftrag nicht ausfhren, weil die bergebenen Parameter falsch sind. So kann der Auftraggeber zum Beispiel nicht die
	 *          ntigen Rechte besitzen einen neuen Benutzer anzulegen.
	 */
	public void createNewUser(
			String orderer, String ordererPassword, String newUsername, String newUserPid, String newPassword, boolean adminRights, String pidConfigurationArea
	) throws ConfigurationTaskException;

	/**
	 * Beauftragt die Konfiguration ein neues Benutzerkonto anzulegen. Zu einem Benutzerkonto gehren ein Benutzername, ein Passwort und die Rechte des
	 * Benutzers.
	 * <p/>
	 * Ein neuer Benutzer kann nur durch einen Benutzer angelegt werden, der die Rechte eines Administrators besitzt. Besitzt der Benutzer diese Rechte nicht,
	 * wird der Auftrag zur Konfiguration bertragen und dort abgelehnt.
	 *
	 * @param orderer              Benutzername des Auftraggebers.
	 * @param ordererPassword      Passwort des Auftraggebers.
	 * @param newUsername          Benutzername des neuen Benutzers.
	 * @param newUserPid           Pid des neuen Benutzers. Wird der Leerstring ("") bergeben, wird dem Benutzer keine explizite Pid zugewiesen.
	 * @param newPassword          Passwort des neuen Benutzers.
	 * @param adminRights          <code>true</code>, wenn der neue Benutzer die Rechte eines Administrators besitzen soll; <code>false</code>, wenn der Benutzer
	 *                             keine speziellen Rechte besitzen soll.
	 * @param pidConfigurationArea Pid des Konfigurationsbereichs, in dem der neue Benutzer angelegt werden soll.
	 * @param data                 Konfigurierende Datenstze mit den dazugehrigen Attributgruppenverwendungen, die fr das neue Benutzer-Objekt gespeichert
	 *                             werden sollen. Oder <code>null</code> falls keine Datenstze angelegt werden sollen.
	 *
	 * @see de.bsvrz.dav.daf.main.config.ConfigurationArea#createDynamicObject(de.bsvrz.dav.daf.main.config.DynamicObjectType, String, String, java.util.Collection)
	 *
	 * @throws de.bsvrz.dav.daf.main.config.ConfigurationTaskException
	 *          Die Konfiguration kann den Auftrag nicht ausfhren, weil die bergebenen Parameter falsch sind. So kann der Auftraggeber zum Beispiel nicht die
	 *          ntigen Rechte besitzen einen neuen Benutzer anzulegen.
	 */
	public void createNewUser(
			String orderer,
			String ordererPassword,
			String newUsername,
			String newUserPid,
			String newPassword,
			boolean adminRights,
			String pidConfigurationArea,
			Collection<DataAndATGUsageInformation> data) throws ConfigurationTaskException;

	/**
	 * Beauftragt die Konfiguration ein Benutzerkonto zu lschen. Alle Informationen, die zu einem Benutzerkonto gehren, sind mit dem Passwort des
	 * Auftraggebers zu verschlsseln und verschlsselt zu bertragen.
	 * <p/>
	 * Ein Benutzerkonto kann nur durch einen Benutzer gelscht werden, der die Rechte eines Administrators besitzt.
	 *
	 * @param orderer              Benutzername des Auftraggebers.
	 * @param ordererPassword      Passwort des Auftraggebers.
	 * @param userToDelete         Benutzername des zu lschenden Benutzers.
	 *
	 * @throws de.bsvrz.dav.daf.main.config.ConfigurationTaskException
	 *          Die Konfiguration kann den Auftrag nicht ausfhren.
	 */
	public void deleteUser(
			String orderer, String ordererPassword, String userToDelete) throws ConfigurationTaskException;

	/**
	 * Prft, ob ein angegebener Benutzer Admin-Rechte besitzt. Jeder Benutzer kann diese Aktion ausfhren.
	 *
	 * @param orderer              Benutzername des Auftraggebers.
	 * @param ordererPassword      Passwort des Auftraggebers.
	 * @param username             Name des zu prfenden Benutzers
	 *
	 * @return true falls der Benutzer Admin-Rechte hat
	 *
	 * @throws de.bsvrz.dav.daf.main.config.ConfigurationTaskException
	 *          Die Konfiguration kann den Auftrag nicht ausfhren.
	 */
	public boolean isUserAdmin(String orderer, String ordererPassword, String username) throws ConfigurationTaskException;
	
	/**
	 * Beauftragt die Konfiguration die Rechte eines Benutzers zu ndern.
	 * <p/>
	 * Nur ein Benutzer mit den Rechten eines Administrators darf die Rechte anderer Benutzer ndern. Besitzt ein Benutzer diese Rechte nicht wird der Auftrag an
	 * die Konfiguration verschickt und dort von der Konfiguration abgelehnt.
	 *
	 * @param orderer         Auftraggeber, der die Rechte eines Benuters ndern mchte.
	 * @param ordererPassword Passwort des Auftraggebers.
	 * @param user            Benutzer, dessen Rechte gendert werden sollen.
	 * @param adminRights     <code>true</code>, wenn der Benutzer, der mit <code>user</code> identifiziert wird, die Rechte eines Administrators erhalten soll;
	 *                        <code>false</code>, wenn der Benutzer, der mit <code>user</code> identifiziert wird, keine speziellen Rechte erhalten soll.
	 *
	 * @throws de.bsvrz.dav.daf.main.config.ConfigurationTaskException Die Konfiguration kann den Auftrag nicht ausfhren, weil die bergebenen Parameter falsch sind. So kann der Auftraggeber
	 *                                    zum Beispiel nicht die ntigen Rechte besitzen die Rechte eines anderen Benutzers zu ndern.
	 */
	public void changeUserRights(String orderer, String ordererPassword, String user, boolean adminRights) throws ConfigurationTaskException;

	/**
	 * Beauftragt die Konfiguration das Passwort eines Benutzers zu ndern. Diese Methode kann von jedem Benutzer aufgerufen werden. Es ist jedem Benutzer
	 * gestattet das Passwort seines Benutzerkontos zu ndern. Soll das Passwort eines fremden Benutzerkontos gendert werden, sind die Rechte eines Administrators
	 * ntig.
	 * <p/>
	 *
	 * @param orderer         Benutzername des Auftraggebers
	 * @param ordererPassword Derzeit gltiges Passwort, falls der Benutzername <code>orderer</code> und <code>user</code> identisch sind. Sind die Parameter nicht
	 *                        identisch, muss der Benutzer, der mit <code>orderer</code> identifiziert wird, die Rechte eines Administrators besitzen und sein
	 *                        Passwort bergeben
	 * @param user            Benutzername des Benutzerkontos, dessen Passwort gendert werden soll
	 * @param newUserPassword Neues Passwort des Benutzers, der mit <code>user</code> identifiziert wurde
	 *
	 * @throws ConfigurationTaskException Die Konfiguration kann den Auftrag nicht ausfhren, weil die bergebenen Parameter falsch sind. So kann der Auftraggeber
	 *                                    zum Beispiel nicht die ntigen Rechte besitzen das Passwort eines anderen Benutzers zu ndern oder das Passwort zum
	 *                                    ndern ist falsch.
	 */
	public void changeUserPassword(String orderer, String ordererPassword, String user, String newUserPassword) throws ConfigurationTaskException;

	/**
	 * Beauftragt die Konfiguration ein Einmal-Passwort zu erzeugen und es einem Benutzer zu zuordnen.
	 * <p/>
	 * Damit dieser Auftrag ausgefhrt werden kann, muss der Auftraggeber <code>orderer</code> die Rechte eines Administrators besitzen. Besitzt der Auftraggeber
	 * diese Rechte nicht, wird der Auftrag zwar zur Konfiguration bertragen, dort aber abgelehnt.
	 *
	 * @param orderer               Benutzername des Auftraggebers
	 * @param ordererPassword       Passwort des Auftraggebers.
	 * @param username              Benutzername, dem ein Einmal-Passwort hinzugefgt werden soll.
	 * @param singleServingPassword Einmal-Passwort das dem Benutzer, der mit <code>username</code> identifiziert wird, hinzugefgt wird.
	 *
	 * @throws de.bsvrz.dav.daf.main.config.ConfigurationTaskException Die Konfiguration kann den Auftrag nicht ausfhren, weil die bergebenen Parameter falsch sind. So kann der Auftraggeber
	 *                                    zum Beispiel nicht die ntigen Rechte besitzen ein Einmal-Passwort anzulegen oder das Passwort existierte bereits, usw..
	 */
	public void createSingleServingPassword(String orderer, String ordererPassword, String username, String singleServingPassword)
			throws ConfigurationTaskException;

	/**
	 * Ermittelt die Anzahl der noch vorhandenen, gltigen Einmal-Passwrter. Jeder Benutzer kann diese Anzahl fr seinen eigenen Account ermitteln, 
	 * fr fremde Accounts sind Admin-Rechte notwendig.
	 *
	 * @param orderer               Benutzername des Auftraggebers
	 * @param ordererPassword       Passwort des Auftraggebers, dies wird zum verschlsseln benutzt und darf nicht mit bertragen werden
	 * @param username              Benutzername, dessen Einmalpasswrter geprft werden sollen.
	 *
	 * @return Anzahl der noch vorhandenen, gltigen Einmal-Passwrter
	 *
	 * @throws ConfigurationTaskException Die Konfiguration kann den Auftrag nicht ausfhren.
	 */
	public int getSingleServingPasswordCount(String orderer, String ordererPassword, String username) throws ConfigurationTaskException;

	/**
	 * Lscht alle Einmalpasswrter fr einen angegebenen Benutzer. Es ist jedem Benutzer
	 * gestattet die Passwrter seines eigenen Accounts zu lschen. Soll ein fremdes Benutzerkonto gendert werden, sind Admin-Rechte ntig.
	 *
	 * @param orderer         Benutzername des Auftraggebers
	 * @param ordererPassword Passwort des Auftraggebers, dies wird zum verschlsseln benutzt und darf nicht mit bertragen werden
	 * @param username        Benutzername, dessen Einmalpasswrter gelscht werden sollen.
	 *
	 * @throws ConfigurationTaskException Die Konfiguration kann den Auftrag nicht ausfhren.
	 */
	public void clearSingleServingPasswords(String orderer, String ordererPassword, String username) throws ConfigurationTaskException;

	/**
	 * Prft, ob ein angegebener Benutzername gltig ist, d.h. ob er ein zugeordnetes Systemobjekt und einen Eintrag in der Benutzerverwaltung.xml hat. Jeder
	 * Benutzer kann diese Aktion ausfhren. Zur (verschlsselten) bertragung des Vorgangs ist dennoch die Angabe eines gltigen Benutzernamens und Passworts
	 * notwendig. Mit dieser Funktion kann geprft werden, ob die Benutzernamen, die {@link #subscribeUserChangeListener(de.bsvrz.dav.daf.main.config.MutableCollectionChangeListener)
	 * } liefert, tatschlichen zu einem gltigen Benutzer gehren, da subscribeUserChangeListener nur die Systemobjekte bercksichtigt.
	 *
	 * @param orderer         Benutzername des Auftraggebers.
	 * @param ordererPassword Passwort des Auftraggebers.
	 * @param username        Name des zu prfenden Benutzers
	 *
	 * @return true falls der Benutzer in der Konfiguration gespeichert ist
	 *
	 * @throws de.bsvrz.dav.daf.main.config.ConfigurationTaskException
	 *          Die Konfiguration kann den Auftrag nicht ausfhren.
	 */
	public boolean isUserValid(String orderer, String ordererPassword, String username) throws ConfigurationTaskException;

	/**
	 * Erstellt einen Listener der nderungen an den Benutzern berwacht. Gibt eine aktuelle Liste aller Benutzer zurck.
	 * @param listener Objekt, an das Rckmeldungen gesendet werden sollen. <code>null</code>, wenn nur die Liste der aktuellen Benutzer geholt werden soll.
	 * @return Liste der aktuell vorhandenen Benutzer. Es ist eventuell ratsam, mit isUserValid zu prfen, ob die Benutzer tatschlich in der
	 * Benutzerverwaltung.xml abgelegt sind, da hier nur die SystemObjekte bercksichtigt werden.
	 */
	public List<SystemObject> subscribeUserChangeListener(MutableCollectionChangeListener listener);

	/**
	 * Entfernt den mit subscribeUserChangeListener erstellten Listener
	 * @param listener Objekt, and das keine Rckmeldungen mehr gesendet werden sollen.
	 */
	public void unsubscribeUserChangeListener(MutableCollectionChangeListener listener);
}
